function [ex, elambda, Hisx, Hisacsize, Hisiter,Hisresidual,Bic] = pathgista(Psi,y,method,opts)

% ========================================================================
% Approximate the  solution of \Psi*x + eta = y
% by mimimizing  1/2||Psi*x-y||^2 + alpha/2||x||^2 + P_{lambda,tao}(x)
% with a unified primal-dual active set  algorithm (UPDAS)combined with 
% pathfollowing and discrepency principle, where the path is 
% Lambda ={lambda_{1},...,lambda_{N}.
% Copyright (c), Dec, 2012
% Yuling Jiao  Dept. Math.,Whuhan Univiversity
% Email: yulingjiaomath@whu.edu.cn
% ========================================================================
% ========================================================================
%% Input:  
% Psi:           the sampling  matrix in R^{n*p} (normalized)
% y:             observation vector vector in R^{n}
% method:        1 for capped-l_1; 2 for log; 3 for scad; 4 for mcp;
%                (defaut: 1)
% opts.          is a structure contained: 
% opts.N:        the size of Lambda (defaut: 100)
% opt.Lamend:    the minimun in Lambda (defaut: 1e-10) 
% opts.noiselev: stop when ||residual||<= opts.noiselev (discrepancy principle) 
% opts.mu:       stop  when ||x_{lambda_{k}}||> opts.mu*n (defaut: 0.5)
%% Output: 
% ex:            an estimate of regression coefficient selected by BIC
% elambda:       regularization parameter selected by BIC
% Hisx:          a matix  storing the solutions on the path 
% Hisacsize:     a vector storing the support size of solutions on the path 
% Hisiter:       a vector storing the iteration number when UPDAS on the path
% Hisresidual:   a vector storing the residual on the path
% Bic:           BIC value on the path 
% ========================================================================
[n,p] = size(Psi);
if nargin < 2
    disp('error, not enough inputment')
elseif nargin == 2
    method = 0;
    opts.alpha = 0;
    opts.N = 100;
    opts.Lamend = 1e-15;
    opts.Maxiter = 10;
    opts.noiselev = 0; % in this it will run untill ||x_{lambda_{k}}||>= opts.mu*n
    opts.mu = 0.5;
    opts.initial = zeros(p,1);
elseif nargin == 3 || isempty(opts)
    opts.alpha = 0;
    opts.N = 100;
    opts.Lamend = 1e-15;
    opts.Maxiter = 10;
    opts.noiselev = 0; % in this it will run untill ||x_{lambda_{k}}||>= opts.mu*n
    opts.mu = 0.5;
    opts.initial = zeros(p,1);
elseif nargin > 4
    disp('error, too much inputments')
end


% define the path Lambda 
lambdaMin = opts.Lamend;
lambdaMax = 1;
loghi = log(lambdaMax);
loglo = log(lambdaMin);
logrange = loghi - loglo;
interval = -logrange/(opts.N-1);
Lambda = exp(loghi:interval:loglo)';

Lambda = Lambda*norm(Psi'*y,inf)/n; % not so exact for log but here we do not use 

 
    
lengthLam = length(Lambda);
Hisx = zeros(p,lengthLam);
Hisacsize = [];
Hisiter =[];
Hisresidual = [];
Bic = [];
%% main loop for pathfolling and choosing lambda and output solution 
if method == 2
    disp(' run Gista Log  on Lambda ...')
   tau = 1.5;
elseif method == 3
   disp(' run Gista SCAD  on Lambda ...')
   tau = 3.7;
elseif method == 4
   disp(' run Gista MCP  on Lambda ...')
   tau = 2.7;
else 
   disp(' run Gista Capped-l_1  on Lambda ...')
   tau = 3.5;
end
    
acsize = 0;
k = 0;
residual = opts.noiselev + 1;
w0 = zeros(p,1);
while acsize <= opts.mu*n && k < lengthLam &&  residual> opts.noiselev
         k = k + 1;
         lambda = Lambda(k); 
         stopcriterion = 1; % stopping criterion (default: 1)
         if method < 3
         maxiter = 1000; % number of maximum iteration (default: 1000)
         else
         maxiter = 20;
         end
         tol = 1e-5; % stopping tolerance (default: 1e-5)
         M = 5; % nonmonotone steps (default: 5)
         t = 1; % initialization of t (default: 1)
         tmin = 1e-20; % tmin parameter (default: 1e-20)
         tmax = 1e20; % tmax parameter (default: 1e20)
         ls = 1e-5; % parameter in the line search (default: 1e-5)
         eta = 2; % eta factor (default: 2)
         stopnum = 3; % number of satisfying stopping criterion (default: 3)
         maxinneriter = 20; % number of maximum inner iteration (line search) (default: 20)
         % call the function
        [ex,fun,time,niter] = gistLeast(Psi,y,lambda,tau,...
                              'maxiteration',maxiter,...
                              'regtype',method,...
                              'stopcriterion', stopcriterion,...
                              'tolerance',tol,...
                              'startingpoint',w0,...
                              'nonmonotone',M,...
                              'tinitialization',t,...
                              'tmin',tmin,...
                              'tmax',tmax,...
                              'sigma',ls,...
                              'eta',eta,...
                              'stopnum',stopnum,...
                              'maxinneriter',maxinneriter);
         w0 = ex;
         Hisx(:,k)= ex;
         Hisiter = [Hisiter;niter];
         Hisacsize = [Hisacsize;length(find(ex))];
         residual = norm(Psi*ex-y);
         Hisresidual = [Hisresidual;residual];
         bick = 0.5*residual^2+ log(n)*acsize;  
         Bic = [Bic;bick];       
end
%% output a solution 
    if residual <= opts.noiselev
    disp('the discrepancy principle is satisfied')
    elambda = Lambda(k);
    ex = Hisx(:,k);
    else
    disp('the discrepancy principle is not satisfied and output a solution by BIC')
    id  = find(Bic == min(Bic));  
    ex = Hisx(:,id(1));
    elambda = Lambda(id(1));
    end



