function [betahat, lambdahat, Hisbeta, HisAclen, HisIter, HisResidual, Bic] = irlspath(X,y,opts)

% ========================================================================
% Approximate the  solution of \Psi*x + eta = y
% by mimimizing  1/2||X*beta-y||^2 + alpha/2||beta||^2 + P_{lambda,tao}(beta)
% with a Multi-stage convex relaxation and PFBS (CD-J,ISTA) combined with pathfollowing on  
% Lambda ={lambda_{1},...,lambda_{N}.
% Copyright (c), Dec, 2012
% Yuling Jiao  Dept. Math.,Whuhan Univiversity
% Email: yulingjiaomath@whu.edu.cn
% ========================================================================
% ========================================================================
% Input:  
% X:           the sampling matrix (nomalized) in R^{n*p}
% y:           observed vector in R^{n}
% opts.method  0, for l_0, 0<q<1 for l_q, 1 for l_1, 2 for SCAD, 3 for MCP,
               % 4 for Capped-l_1  (defult: l_1) 
% opts.stage:  Maximum number of MSCR iteations. (defuat:5)
% opts.stagestop:  stop when relative  stage change smaller than it (default,1e-5) 
% opts. alpha:     to handle ill conditon case (default: 0)
% opts.N:      the size of Lambda  (defaut: 100)
% opts.J:      maxiteration number in PFBS  (defaut: 5*n)
% opts.mu:     stop parameter when ||beta(lambda_k)||_{0}>mu*n  (defaut: 0.5)
% opts.Lamend: Lambda = norminf*[1,Lamend] (defaut: 1e-10) 
% opts.normnX: step length that make convergence (default norm(nX))
% opts.initial: initial value, (default(zeros(p,1)))
% opts.stop:    stop when norm(beta^k+1)/norm(beta^k+1-beta^k)<opts.stop 
%               (default,1e-5)
   
% Output: 
% betahat:      an estimate of beta
% lambdahat:    regularization parameter 
% Hisbeta:      a matix  storing the history of 
%               beta_{lambda_{k}},k =1,...N 
% HisAClen:     a vector storing the history of acive set size of
%               beta_{lambda_{k}},k =1,...N 
% HisIter:      a vector storing  the iteration number when  stop at
%               each  lambda_{k},k =1,...N
% HisResidual:  Residual ateach  lambda_{k},k =1,...N              
% Bic:          BIC value 
% ========================================================================
[n,p] = size(X);
if nargin < 2
    disp('error, not enough inputment ')
elseif  nargin > 3
    disp('error, too much inputment ')
elseif nargin == 2 || isempty(opts)
    opts.method = 1;
    opts.stage = 5;
    opts.stagestop = 1e-5; 
    opts.N = 100;
    opts.tau = 1e-10;
    opts.J = 5*n;
    opts.mu = 0.5;
    opts.normnX = norm(nX);
    opts.initial = zeros(p,1);
    opts.stop = 1e-5;
end


% define the set Lambda
opts.Xty = X'*y;
lambdaMin = opts.Lamend;
lambdaMax = 1;
loghi = log(lambdaMax);
loglo = log(lambdaMin);
logrange = loghi - loglo;
interval = -logrange/(opts.N-1);
Lambda = exp(loghi:interval:loglo)';
Lambda = Lambda(2:end);
Lambda = norm(opts.Xty,inf)*Lambda;
lenLam = length(Lambda);

Hisbeta = zeros(p,lenLam);
HisAclen = [];
HisIter =[];
Bic = [];
HisResidual = [];
%% main loop for choosing lamda 


k = 0;
change =  opts.change + 1;
obeta = opts.initial;
while  k < lenLam && change > opts.change
         k = k + 1;
         lambda = Lambda(k);
         for kk = 1:opts.stage
             kk;
         W = lambda./(opts.initial.^2 + opts.alpha).^((2-opts.method)/2);
         rG = opts.G + spdiags(W,0,p,p);
         ebeta = rG\opts.Xty;
         opts.initial = ebeta; % warm starting 
         end
         residual = norm(X*ebeta-y);
         Hisbeta(:,k) = ebeta;
         HisResidual = [HisResidual;residual];
        change = norm(obeta - ebeta)/(norm(obeta)+opts.alpha);
        obeta = ebeta;
end

% select the optimal betahat and lambdahat  
id  = find(HisResidual == min(HisResidual));
lambdahat = Lambda(id(1));
betahat = Hisbeta(:,id(1));
end