function [betahat, lambdahat, Hisbeta, HisAclen, HisIter, HisResidual, Bic] = mscrpfbspath(X,y,opts)

% ========================================================================
% Approximate the  solution of \Psi*x + eta = y
% by mimimizing  1/2||X*beta-y||^2 + alpha/2||beta||^2 + P_{lambda,tao}(beta)
% with a Multi-stage convex relaxation and PFBS (CD-J,ISTA) combined with pathfollowing on  
% Lambda ={lambda_{1},...,lambda_{N}.
% Copyright (c), Dec, 2012
% Yuling Jiao  Dept. Math.,Whuhan Univiversity
% Email: yulingjiaomath@whu.edu.cn
% ========================================================================
% ========================================================================
% Input:  
% X:           the sampling matrix (nomalized) in R^{n*p}
% y:           observed vector in R^{n}
% opts.method  0, for l_0, 0<q<1 for l_q, 1 for l_1, 2 for SCAD, 3 for MCP,
               % 4 for Capped-l_1  (defult: l_1) 
% opts.stage:  Maximum number of MSCR iteations. (defuat:5)
% opts.stagestop:  stop when relative  stage change smaller than it (default,1e-5) 
% opts. alpha:     to handle ill conditon case (default: 0)
% opts.N:      the size of Lambda  (defaut: 100)
% opts.J:      maxiteration number in PFBS  (defaut: 5*n)
% opts.mu:     stop parameter when ||beta(lambda_k)||_{0}>mu*n  (defaut: 0.5)
% opts.Lamend: Lambda = norminf*[1,Lamend] (defaut: 1e-10) 
% opts.normnX: step length that make convergence (default norm(nX))
% opts.initial: initial value, (default(zeros(p,1)))
% opts.stop:    stop when norm(beta^k+1)/norm(beta^k+1-beta^k)<opts.stop 
%               (default,1e-5)
   
% Output: 
% betahat:      an estimate of beta
% lambdahat:    regularization parameter 
% Hisbeta:      a matix  storing the history of 
%               beta_{lambda_{k}},k =1,...N 
% HisAClen:     a vector storing the history of acive set size of
%               beta_{lambda_{k}},k =1,...N 
% HisIter:      a vector storing  the iteration number when  stop at
%               each  lambda_{k},k =1,...N
% HisResidual:  Residual ateach  lambda_{k},k =1,...N              
% Bic:          BIC value 
% ========================================================================
[n,p] = size(X);
if nargin < 2
    disp('error, not enough inputment ')
elseif  nargin > 3
    disp('error, too much inputment ')
elseif nargin == 2 || isempty(opts)
    opts.method = 1;
    opts.stage = 5;
    opts.stagestop = 1e-5; 
    opts.N = 100;
    opts.tau = 1e-10;
    opts.J = 5*n;
    opts.mu = 0.5;
    opts.normnX = norm(nX);
    opts.initial = zeros(p,1);
    opts.stop = 1e-5;
end


% define the set Lambda
opts.Xty = X'*y;
lambdaMin = opts.Lamend;
lambdaMax = 1;
loghi = log(lambdaMax);
loglo = log(lambdaMin);
logrange = loghi - loglo;
interval = -logrange/(opts.N-1);
Lambda = exp(loghi:interval:loglo)';
Lambda = Lambda(2:end);
Lambda = norm(opts.Xty,inf)*Lambda;
lenLam = length(Lambda);

Hisbeta = zeros(p,lenLam);
HisAclen = [];
HisIter =[];
Bic = [];
HisResidual = [];
%% main loop for choosing lamda 


mun = opts.mu*n;
if opts.method == 1 
    stage =  opts.stage - 1;
else
    stage = 0;
end
change = opts.stagestop + 1;
obeta = opts.initial;
residual = opts.noiselev + 1;
while stage < opts.stage  &&change > opts.stagestop 
    %&& residual > opts.noiselev  % if this stoping rule is used it always
    %stop in one stage, since the soluion is initialized with lasso which
    %satisfied the discrepancy principle.
    stage = stage + 1
    disp(' run PFBS on Lambda ...')
    opts.initial = obeta;
    k = 0;
    df = 0;
    Hisbeta = zeros(p,lenLam);
    HisAclen = [];
    HisIter =[];
    Bic = [];
    HisResidual = [];
while df <mun && k < lenLam 
         k = k + 1;
         lambda = Lambda(k);
         W =  getgrad(obeta,opts.method,lambda);
         [ebeta, AClen, AC, nIter] = pfbswl1(X,y,lambda,W,opts);
         Hisbeta(:,k) = ebeta;
         HisIter = [HisIter;nIter];
         df = AClen;
         HisAclen = [HisAclen;df];
         opts.initial = ebeta; % warm starting 
         residual = norm(X*ebeta-y);
         HisResidual = [HisResidual;residual];
         bick = 0.5*residual^2+log(n)*df;% Bic
         Bic = [Bic;bick];        
end

% select the optimal betahat and lambdahat  
id  = find(Bic == min(Bic));
lambdahat = Lambda(id(1));
betahat = Hisbeta(:,id(1));
change = norm(obeta-betahat)/(1e-8+norm(obeta));
obeta = betahat;
end