function [betahat, lambdahat, Hisbeta, HisAclen, HisIter, Bic] = pfbsl1path(X,y,opts)
% ========================================================================
% Approximate the  solution path of L_1 regularized  High dimentional 
% problem y = X*beta + eta with n << p by minmizing 
%     1/2||X*beta-y||^2  + lambda||beta||_{1} 
% with PFBS on  Lambda ={lambda_{1},...,lambda_{N}}
% Copyright (c), Des, 2012
% Yuling Jiao  Dept. Math.,Whuhan Univiversity
% Email: yulingjiaomath@whu.edu.cn
% ========================================================================
% ========================================================================
% Input:  
% X:            the sampling matrix (nomalized) in R^{n*p}
% y:            observed vector in R^{n}
% opts.N:       the size of Lambda  (defaut: 100)
% opts.J:       maxiteration number in PFBS for safe guard   (defaut: 5*n)
% opts.mu:      stop parameter when ||beta(lambda_k)||_0  > mu*n  (defaut: 0.5)
% opts.Lamend:  Lambda = norminf*[Lamend,1] (defaut: 1e-10-1e-15) 
% opts.normnX:  step length that make convergence (default norm(nX))
% opts.initial: initial value, (default(zeros(p,1)))
% opts.stop:    stop when norm(beta^k+1)/norm(beta^k+1-beta^k)<opts.stop 
%               (default,1e-6)
   
% Output: 
% betahat:      an estimate of beta 
% lambdahat:    regularization parameter 
% Hisbeta:      a matix  storing the history of 
%               beta_{lambda_{k}},k =1,...N 
% HisAClen:     a vector storing the history of acive set size of
%               beta_{lambda_{k}},k =1,...N 
% HisIter:      a vector storing  the iteration number when SSN stop at
%               each  lambda_{k},k =1,...N
% Bic:          BIC value 
% ========================================================================
[n,p] = size(X);
if nargin < 2
    disp('error, not enough inputment ')
elseif  nargin > 3
    disp('error, too much inputment ')
elseif nargin == 2 || isempty(opts)
    opts.N = 100;
    opts.lamend = 1e-15;
    opts.J = 5*n;
    opts.mu = 0.5;
    opts.normnX = norm(nX);
    opts.initial = zeros(p,1);
    opts.stop = 1e-6;
end


% define the set Lambda 
opts.Xty = X'*y;
lambdaMin = opts.lemend;
lambdaMax = 1;
loghi = log(lambdaMax);
loglo = log(lambdaMin);
logrange = loghi - loglo;
interval = -logrange/(opts.N-1);
Lambda = exp(loghi:interval:loglo)';
Lambda = Lambda(2:end);
Lambda = norm(opts.Xty,inf)*Lambda;
lenLam = length(Lambda);


Hisbeta = zeros(p,lenLam);
HisAclen = [];
HisIter =[];
Bic = [];

%% main loop for pathfollowing PFBS 
disp(' run PFBS on Lambda ...')
df = 0;
k = 0;
mun = opts.mu*n;

while df <mun && k < lenLam
         k = k + 1;
         lambda = Lambda(k);
         w = lambda*ones(p,1);
         [ebeta, AClen, AC, nIter] = pfbswl1(X,y,lambda,w,opts);
         Hisbeta(:,k) = ebeta;
         HisIter = [HisIter;nIter];
         df = AClen;
         HisAclen = [HisAclen;df];
         opts.initial = ebeta;
         bick = 0.5*norm(X*ebeta-y)^2+log(n)*df;% Bic
         Bic = [Bic;bick];        
end

% select the optimal betahat and lambdahat  
id  = find(Bic == min(Bic));
lambdahat = Lambda(id(1));
betahat = Hisbeta(:,id(1));



