function [ex, elambda, Hisx, Hisacsize, Hisiter,Hisresidual,Bic] = pathdiscnonconvex(Psi,y,method,opts)

% ========================================================================
% Approximate the  solution of \Psi*x + eta = y
% by mimimizing  1/2||Psi*x-y||^2 + alpha/2||x||^2 + P_{lambda,tao}(x)
% with a unified primal-dual active set  algorithm (UPDAS)combined with 
% pathfollowing and discrepency principle, where the path is 
% Lambda ={lambda_{1},...,lambda_{N}.
% Copyright (c), Dec, 2012
% Yuling Jiao  Dept. Math.,Whuhan Univiversity
% Email: yulingjiaomath@whu.edu.cn
% ========================================================================
% ========================================================================
%% Input:  
% Psi:           the sampling  matrix in R^{n*p} (normalized)
% y:             observation vector vector in R^{n}
% method:        q in [0,1] for l_{q}; 2 for scad; 3 for mcp; 4 for capped-l_1
%                (defaut: 0)
% opts.          is a structure contained: 
% opts.alpha :   a parameter for ill-conditioned Psi  (defaut: 0) which 
%                should be carefully given.
% opts.N:        the size of Lambda (defaut: 100)
% opt.Lamend:    the minimun in Lambda (defaut: 1e-15) 
% opts.noiselev: stop when ||residual||<= opts.noiselev (discrepancy principle) 
% opts.mu:       stop  when ||x_{lambda_{k}}||> opts.mu*n (defaut: 0.5)
% opts.initial:  initial value for UPDAS (defaut: 0)
% opts.Maxiter:  maximum number of iteration in UPDAS  (defaut: 10)
%% Output: 
% ex:            an estimate of regression coefficient selected by BIC
% elambda:       regularization parameter selected by BIC
% Hisx:          a matix  storing the solutions on the path 
% Hisacsize:     a vector storing the support size of solutions on the path 
% Hisiter:       a vector storing the iteration number when UPDAS on the path
% Hisresidual:   a vector storing the residual on the path
% Bic:           BIC value on the path 
% ========================================================================
[n,p] = size(Psi);
if nargin < 2
    disp('error, not enough inputment')
elseif nargin == 2
    method = 0;
    opts.alpha = 0;
    opts.N = 100;
    opts.Lamend = 1e-15;
    opts.Maxiter = 10;
    opts.noiselev = 0; % in this it will run untill ||x_{lambda_{k}}||>= opts.mu*n
    opts.mu = 0.5;
    opts.initial = zeros(p,1);
elseif nargin == 3 || isempty(opts)
    opts.alpha = 0;
    opts.N = 100;
    opts.Lamend = 1e-15;
    opts.Maxiter = 10;
    opts.noiselev = 0; % in this it will run untill ||x_{lambda_{k}}||>= opts.mu*n
    opts.mu = 0.5;
    opts.initial = zeros(p,1);
elseif nargin > 4
    disp('error, too much inputments')
end


% define the path Lambda 
lambdaMin = opts.Lamend;
lambdaMax = 1;
loghi = log(lambdaMax);
loglo = log(lambdaMin);
logrange = loghi - loglo;
interval = -logrange/(opts.N-1);
Lambda = exp(loghi:interval:loglo)';

if method == 3 
     Lambda = Lambda(4:end);
else
     Lambda = Lambda(2:end);
end

if method == 0
Lambda = Lambda*(norm(Psi'*y,inf))^2/2;
elseif 0<method<1
    Lambda = Lambda*(norm(Psi'*y,inf)/(2-method)*(2*(1-method))^((method-1)/(2-method)))^(2-method);
else
   Lambda = Lambda*norm(Psi'*y,inf);
end
 
    
lengthLam = length(Lambda);
Hisx = zeros(p,lengthLam);
ex = zeros(p,1);
Hisacsize = [];
Hisiter =[];
Hisresidual = [];
Bic = [];
%% main loop for pathfolling and choosing lambda and output solution 
if length(struct2cell(opts)) == 8
if method == 0
   disp(' run UPDAS l_0 on Lambda ...')
   tau = [];
elseif 0 <method && method < 1
   disp(' run UPDAS Bridge on Lambda ...')
   tau = method;
elseif method == 1
   disp(' run UPDAS (SSN) l_1  on Lambda ...')
   tau = [];
elseif method == 2
   disp(' run UPDAS SCAD  on Lambda ...')
   tau = 3.7;
elseif method == 3
   disp(' run UPDAS MCP  on Lambda ...')
   tau = 2.7;
elseif method == 4
   disp(' run UPDAS Capped-l_1  on Lambda ...')
   tau = 1.5;
end
else
    disp('\tau is input by the user')
    tau = opts.tau
end
    
acsize = 0;
k = 0;
residual = opts.noiselev + 1;
initial = opts.initial;
alpha = opts.alpha;
maxiter = opts.Maxiter;
mu = opts.mu;
while acsize <= opts.mu*n && k < lengthLam &&  residual> opts.noiselev
         k = k + 1;
         lambda = Lambda(k);
         if opts.testkind == 2
         [ex,acsize,xhis,niter] = updasreal(Psi,y,alpha,lambda,tau,method,initial,maxiter,mu);
         else
         [ex,acsize,xhis,niter] = updas(Psi,y,alpha,lambda,tau,method,initial,maxiter,mu);
         end
         initial = ex;
         Hisx(:,k)= ex;
         Hisiter = [Hisiter;niter];
         Hisacsize = [Hisacsize;acsize];
         residual = norm(Psi*ex-y);
         Hisresidual = [Hisresidual;residual];
         if opts.testkind == 2  % for real data
           bick = log(residual^2/(n-acsize)) + log(n)*acsize/n;
         else
           bick = 0.5*residual^2+ log(n)*acsize;
         end   
         Bic = [Bic;bick];       
end
%% output a solution 
    if residual <= opts.noiselev
    disp('the discrepancy principle is satisfied')
    elambda = Lambda(k);
    ex = Hisx(:,k);
    else
    disp('the discrepancy principle is not satisfied and output a solution by BIC')
    id  = find(Bic == min(Bic));  
    ex = Hisx(:,id(1));
    elambda = Lambda(id(1));
    end



