function [x,d,s,nIt] = pdas(X,y,opts,x0)
%-------------------------------------------------------------------------%
% Solving nonconvex problem                                               %
%      1/2||X*x-y||^2 + alpha/2||x||^2 + rho_{lam,tau}(x)                 %
% by a primal-dual active set algorithm                                   %
%                                                                         %
% INPUTS:                                                                 %
%     X   ---  matrix, normalized                                         %
%     y   ---  data vector                                                %
%    opts --- structure contains                                          %
%         alpha ---  regularization parameter (default: 0)                %
%          lam  ---  regualrization paramter for sparsity                 %
%        method ---  nonconvex model                                      %
%          tau  ---  concavity parameter                                  %
%           mu  ---  stopping parameter                                   %
%        MaxIt  ---  maximum number of iterations (default: 5)            %
%     x0  ---  initial guess (default: 0)                                 %
% OUTPUTS:                                                                %
%      x  ---  solution                                                   %
%      s  ---  size of active set                                         %
%     nIt ---  # of iter for the PDAS stop                                %
% (c) by Yuling Jiao (yulingjiaomath@whu.edu.cn)                          %
%         Bangti Jin (bangti.jin@gmail.com)                               %
% Created on March 17, 2013                                               %
%-------------------------------------------------------------------------%
[n,m] = size(X);
lam = opts.lam;
tau = opts.tau;
al  = opts.alpha;
switch opts.method
    case 'l0'
        T = sqrt(2*lam);
    case 'bridge'
        T = (2-tau)*(2*(1-tau))^((tau-1)/(2-tau))*lam^(1/(2-tau));
    otherwise
        T = lam;
end

% initializing ...
Xty= X'*y;
p  = x0 + Xty - X'*(X*x0);
x  = x0;
A  = find(abs(p)>T);
pt = abs(p)>T;
d=Xty - X'*(X*x0);
for nIt = 1:opts.MaxIt
    di=d;
    getd  = getdual(x,di,p,opts,A);
    Xa = X(:,A);
    rhs= Xty(A)-getd;
    G = Xa'*Xa+al*eye(length(A));
    x = zeros(m,1);
    x(A) = G\rhs;
    d = X'*(y - Xa*x(A));
    p = x + d;
    A = find(abs(p)>T);
    tp= pt;
    pt= abs(p)>T;
    if length(A)>opts.mu*m
        % too many nonzeros
        break;
    end
    if (pt==tp)
        % active set coincides
        break;
    end
end
s = length(A);