function [x,lam,ithist] = pdas_path(X,y,opts)
%-----------------------------------------------------------------------%
% minimize 1/2||X*x-y||^2 + alpha/2||x||^2 + rho_{lambda,tau}(x)        %
% with PDAS algorithm, stopping criterion: discrepency principle        %
% Lam ={lam_{1},...,lam_{N}}.                                           %
%                                                                       %
% INPUTS:                                                               %
%     X  ---  sampling matrix (normalized)                              %
%     y  ---  data vector                                               %
%   opts ---  structure containing                                      %
%        tau  --- concavity parameter                                   %
%       alpha --- regularization parameter (defaut: 0)                  %
%         N   --- length of path (defaut: 100)                          %
%       Lmin  --- minimun in Lam (defaut: 1e-15)                        %
%        del  --- noise level                                           %
%        mu   --- stop if ||x_{lambda_{k}}||> opts.mu*n (defaut: 0.5)   %
%        x0   --- initial value for PDAS (defaut: 0)                    %
%     MaxIter --- maximum number of iterations in PDAS  (defaut: 5)     %
% OUTPUTS:                                                              %
%     x   ---- reconstructed signal                                     %
%    lam  ---- regu. parameter (by discrepancy principle or BIC)        %
%  ithist ---- structure on iteration history, containing               %
%          .x  --- solution path                                        %
%          .as --- size of active set                                   %
%          .it --- # of iteration on the path                           %
%          .res--- residual on the path                                 %
% (c) by Yuling Jiao (yulingjiaomath@whu.edu.cn)                        %
%         Bangti Jin (bangti.jin@gmail.com)                             %
% Created on March 17, 2013                                             %
% ======================================================================%

[n,p] = size(X);
x0    = zeros(p,1);

if ~exist('opts','var'), opts = set_pdasopts('l0'); end
method = opts.method;

switch method
    case 'l0'
        disp('l_0 model is running ...')
    case 'bridge'
        opts.tau = 0.5;
         disp('Bridge model is running ...')
    case 'scad'
        opts.tau = 3.7;
         disp('Scad model is running ...')
    case 'mcp'
        opts.tau = 2.7;
         disp('Mcp model is running ...')
    case 'cl1'
        opts.tau = 1.5;
         disp('Capped-l_1 model is running ...')
    otherwise
      error('Undefined method in pdas!')
end
% construct the continuation path
Lam   = exp(linspace(log(opts.Lmax),log(opts.Lmin),opts.N))';
Lam   = Lam(2:end);
linf  = norm(X'*y,inf);
switch method
    case 'l0'
        cnst = linf^2/2;
    case 'bridge'
        tau  = opts.tau;
        cnst = (linf/(2-tau)*(2*(1-tau))^((tau-1)/(2-tau)))^(2-tau);
    otherwise
        cnst = linf;
end
Lam = Lam*cnst;

%% main loop for pathfolling and choosing lambda and output solution
ithist.x   = [];
ithist.as  = [];
ithist.it  = [];
ithist.res = [];
ithist.bic = [];
 for k = 1:length(Lam)
    
    opts.lam = Lam(k);
    [x,d,s,nIt] = pdas(X,y,opts,x0);
    x0 = x;
    ithist.x(:,k)= x;
   
    ithist.it = [ithist.it;nIt]; % iter #
 
    ithist.as = [ithist.as;s];% size of active set
  
    res       = norm(X*x-y);                
    ithist.res= [ithist.res; res];
    
    ithist.bic= [ithist.bic; 0.5*res^2 + log(n)*s];
    if res <= opts.del
        % discrepancy principle
        disp('Discrepancy principle is satisfied.')
        lam = Lam(k);
        x   = ithist.x(:,k);
        break;
    end
     if s > opts.mu*n
        display('Too many nonzeros in the reconstruction ....')
        lam = Lam(k);
        break
    end
end
if res > opts.del
    switch method
        case 'bridge'
            %  the current version of BIC does not work well on bridge ...
          
            lam = Lam(k-2);
            x   = ithist.x(:,k-2);
        otherwise
            Bayesian information criterion
            ii = find(ithist.bic == min(ithist.bic),1);
            x  = ithist.x(:,ii);
            lam= Lam(ii);
    end
end