%------------------------------------------------------------------------%
% This is the demo script for the primal dual active set algorithm for   %
% a class of nonconvex sparsity optimization problems of the form        %
%  1/2\|X x -y \|^2 + \rho_{\lambda,\tau}(x)                             %
% It covers the following five penalties: l0, bridge, capped l1, SCAD and%
% MCP. This demo file includes four examples, which are taken from the   %
% companion paper:                                                       %
% Y. Jiao, B. Jin, X. Lu. A primal dual active set algorithm for a class %
% of nonconvex sparsity optimization, preprint, arXiv 1310.1147.         %
%                                                                        %
% (c) by Yuling Jiao (yulingjiaomath@whu.edu.cn)                         %
%         Bangti Jin (bangti.jin@gmail.com)                              %
% Created on March 17, 2013                                              %
%------------------------------------------------------------------------%

clc, clear
close all
addpath(genpath(fileparts(mfilename('fullpath'))));
warning off

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% for your own data you should first normized the column X                %
% by [X,D] = normalize(X);  and run [x,lam,ithist] = pdas_path(X,y,opts)  %
% Then the soulution is   D*x                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%---- small scale CS problem ----- %
p = 1000;           % length of signal x  
n = 300;            % number of samples 
K = 50;             % number of nonzero elements in x
ratio = 2;          % range of value in x (10^ratio)
sigma = 0.1;        % noise standard deviation
kind  = 1;          % standard CS setting
seednum = 0;        % the seed number
[X,D,y,ye,xe,Ae] = gendata(n,p,K,sigma,ratio,seednum,kind);

%---- medium-scale CS problem ----- %
% p = 10000;          % length of solution  
% n = 500;            % number of samples 
% K = 50;             % number of nonzero elements 
% ratio = 3;          % range of value in x = 10^ratio;
% sigma = 0.5;        % noise standard deviation
% kind = 1;           % standard CS setting
% seednum = 0;        % seed number                       
% [X,D,y,ye,xe,Ae] = gendata(n,p,K,sigma,ratio,seednum,kind);

%---- medium-scale Variable Selection problem with correlation --%
% p = 10000;          % length of signal
% n = 1000;           % number of samples 
% K = 100;            % number of nonzero elements
% ratio = 3;          % range of value in x = 10^ratio;
% sigma = 0.05;       % noise standard deviation
% kind = 3;           % CS setting with correlation
% seednum = 0;        % seed number 
% [X,D,y,ye,xe,Ae] = gendata(n,p,K,sigma,ratio,seednum,kind);

%---- Heaviside ~ ill-posed problem ----- %
% p = 50;             % length of the signal
% n = 50;             % number of samples 
% K = 5;              % number of nonzero elements in the signal
% ratio = 2;          % range of value in beta true = 10^ratio;
% sigma = 1e-2;       % noise standard deviation
% kind = 2;           % Heavide matrix, ill-conditioned, requires LARGE opts.mu
% seednum = 0;        % seed number 
% [X,D,y,ye,xe,Ae] = gendata(n,p,K,sigma,ratio,seednum,kind);

%% set default parameters
method  = 'l0';  % other choices: 'bridge', 'scad', 'mcp', 'cl1'
opts = set_pdasopts(method);
opts.del = norm(y-ye);
if kind == 2
    opts.mu = 1;
end
tic, [x,lam,ithist] = pdas_path(X,y,opts); toc
x = D*x;
% compute error and disp the result
err_l2  = norm(x-xe)/norm(xe);
err_linf= norm(x-xe,inf);
A       = find(x);
lenthA  = length(A);
display(sprintf('regu. para. = %g, size of active set = %g, rel. l_2 error = %g, abs. l_inf error = %g',...
lam,lenthA,err_l2, err_linf))

%% display results
% showing the support recovery
% figure(1), plot(1:p,xe,'ko',1:p,x,'r*'), axis square
% legend('exact', 'estimate','Location','southeast')

% showing the convergence behavior of the PDASC
II = length(ithist.it);
AmAe = zeros(II,1);
AemA = zeros(II,1);
for k = 1:II
    AmAe(k) = length(setdiff(find(ithist.x(:,k)),Ae));
    AemA(k) = length(setdiff(Ae,find(ithist.x(:,k))));
end
figure(21),
plot(1:II,AmAe,'--ro',1:II,AemA,'--b*','LineWidth',2),
h = legend('$|A\setminus A^{k}|$','$|A^{k}\setminus A|$',2,'Location','Northeast');
set(h,'Interpreter','latex','fontsize',40)
% legend('|A^{k}-A|','|A-A^{k}|',2,'Location','Northeast')
xlabel('k','FontSize',30), set(gca,'FontSize',20)

figure(22),
plot(ithist.it,'r+','LineWidth',2),
xlabel('k','FontSize',16),set(gca,'FontSize',16)

% showing the residual on the path 
% figure(3), semilogy(1:II, ithist.res,'r*','LineWidth',2), axis square
% set(gca,'FontSize',16)
% title('Residual on the path')